{Written 2020 by Jem Miller @ Missing Link Software
jem@mlsoft.org

Released into the wild in the hopes that it may help others out there that
are trying to maintain an android SDK. This software is licensed under our
WTF license, that is, you can do WTF you want with it.

If you make cool additions to it I'd like a copy, but that is not a requirement.
This was written for my own use to use the google command line tools instead
of having to install the bloated android studio to do the same thing.
}
unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fileutil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  Grids, Buttons, process;

const
     callList = '--list --verbose';
     uninstall = '--uninstall ';
     updates = '--update ';
     license = '--license';
     obsolete = '--include_obsolete';


type

  config = record
    sdkdir,
    ndkdir,
    jredir : shortstring;
  end;

  { Tmainfrm }

  Tmainfrm = class(TForm)
    helpbtn: TBitBtn;
    updatebtn: TBitBtn;
    refreshbtn: TBitBtn;
    savebtn: TBitBtn;
    installbtn: TBitBtn;
    opendlg: TSelectDirectoryDialog;
    uninstbtn: TBitBtn;
    closebtn: TBitBtn;
    progress: TMemo;
    sdkpath: TEdit;
    ndkpath: TEdit;
    jrepath: TEdit;
    pan: TPanel;
    Panel2: TPanel;
    getsdkbtn: TSpeedButton;
    getndkbtn: TSpeedButton;
    getjrebtn: TSpeedButton;
    pgrid: TStringGrid;
    lst : tstringlist;
    sel : tstringlist;
    procedure helpbtnClick(Sender: TObject);
    procedure updatebtnClick(Sender: TObject);
    procedure pgridClick(Sender: TObject);
    procedure refreshbtnClick(Sender: TObject);
    procedure getjrebtnClick(Sender: TObject);
    procedure getndkbtnClick(Sender: TObject);
    procedure getsdkbtnClick(Sender: TObject);
    procedure savebtnClick(Sender: TObject);
    procedure uninstbtnClick(Sender: TObject);
    procedure installbtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
  procedure runcmd(ctype, pick : integer);
  procedure getlist;
  procedure showList;
  procedure unmark(ct : integer);
  procedure mark(ct : integer);
  procedure MarkInstalled(ct : integer);
  procedure checkupdates;
  public

  end;

var
  mainfrm: Tmainfrm;
  crec : config;
  tmp,
  tmp1,
  mydir : string;
  i,x : integer;

procedure getcfg;
procedure savecfg;

implementation
uses
    jstr32, warning, help;

{$R *.lfm}
var
  inst : array[0..100] of string;

procedure Tmainfrm.runcmd(Ctype, pick : integer);
var
  cmd,
  opt,
  OutStr : string;
  ct,
  rt,
  stat : integer;

begin

     case ctype of
     0: begin
     tmp:= sel[pick];
     ct:= s2int(stripto(tmp,','));
     cmd:= '';
     opt := replace(sel[pick],' ',';');
     warnfrm.msg.Caption:= installing;
     progress.Append(pgrid.Cells[0,pick]);
     warnfrm.show;
     end;

     1: begin
     tmp:= sel[pick];
     ct:= s2int(stripto(tmp,','));
     cmd:= uninstall;
     opt:= replace(sel[pick],' ',';');
     warnfrm.msg.Caption:= uninstalling;
     progress.Append(pgrid.Cells[0,pick]);
     warnfrm.show;
     end;

     2: begin
     cmd:= updates + obsolete;
     opt:= '';
     warnfrm.msg.Caption:= _updating;
     warnfrm.show;
     end;

     end;

     application.ProcessMessages;

     rt:= RunCommandInDir(sdkpath.Text,'c:\windows\system32\cmd.exe',
     ['/c', crec.sdkdir + 'tools\bin\sdkmanager.bat ' + cmd + opt,'--sdk_root=' + crec.sdkdir],
     outstr, stat,[poWaitOnExit,poNoConsole]);

     if rt <> 0 then
     begin
     warnfrm.Close;
     showmessage('Could not get the package(s) selected. Check your internet connection and your SDK path.');
     exit;
     end;

     warnfrm.close;
     progress.Append(outstr);

     case ctype of
     0,1 : unmark(ct);
     end;

end;

procedure Tmainfrm.getlist;
var
  OutStr : string;
  rt,
  stat : integer;

begin
     savecfg;

     if crec.sdkdir = '' then
     exit;

     rt:= RunCommandInDir(crec.sdkdir,'c:\windows\system32\cmd.exe',
     ['/c',crec.sdkdir + 'tools\bin\sdkmanager.bat --list --verbose > lst.txt'],
     outstr, stat,[poWaitOnExit,poNoConsole]);

     if rt <> 0 then
     begin
     showmessage('Could not get the package list. Check your internet connection and your SDK path.');
     exit;
     end;

     if lst = nil then
     lst:= tstringlist.Create;

     lst.Clear;
     progress.Clear;

     tmp:= crec.sdkdir + 'lst.txt';

     lst.LoadFromFile(tmp);
     deletefile(crec.sdkdir + 'lst.txt');

     showlist;

end;

{ Tmainfrm }

procedure Tmainfrm.FormShow(Sender: TObject);
begin
     getcfg;

     sdkpath.Text:= crec.sdkdir;
     ndkpath.Text:= crec.ndkdir;
     jrepath.Text:= crec.jredir;

     getlist;

end;

procedure Tmainfrm.installbtnClick(Sender: TObject);
var
   i : integer;

begin
     if (sel = nil) or (sel.Count = 0) then
     exit;

     for i:= 0 to sel.Count - 1 do
     runcmd(0,i);

     sel.Clear;
end;

procedure Tmainfrm.uninstbtnClick(Sender: TObject);
var
   i : integer;

begin
     if (sel = nil) or (sel.Count = 0) then
     exit;

     for i:= 0 to sel.Count - 1 do
     runcmd(1,i);

     sel.Clear;
end;

procedure Tmainfrm.refreshbtnClick(Sender: TObject);
begin
     getlist;
end;

procedure Tmainfrm.unmark(ct : integer);
begin

     if ct = 0 then
        exit;

     if first(1,pgrid.cells[0,ct]) = '>' then
     begin
     tmp:= pgrid.cells[0,ct];
     delete(tmp,1,1);
     pgrid.cells[0,ct]:= tmp;
     sel.Delete(sel.IndexOf(i2str(ct) + ',' + tmp));
     end;

end;

procedure Tmainfrm.mark(ct : integer);
begin
     tmp:= strip(pgrid.cells[0,ct],'*');
     sel.Append(i2str(ct) + ',' +tmp);

     tmp:= '>' + tmp;
     pgrid.cells[0,ct]:= tmp;

end;

procedure Tmainfrm.pgridClick(Sender: TObject);
var
   ct : integer;

begin
     ct:= pgrid.Row;

     if (pgrid.cells[0,ct] = 'Available Packages:') or
        (pgrid.cells[0,ct] = '') or
        (ct = 0) then
        exit;

     if sel = nil then
     sel:= tstringlist.Create;

     if first(1,pgrid.cells[0,ct]) = '>' then
     unmark(ct)
     else
     mark(ct);

end;

procedure Tmainfrm.updatebtnClick(Sender: TObject);
begin
     runcmd(2,i);
end;

procedure Tmainfrm.helpbtnClick(Sender: TObject);
begin
     helpfrm.show;
end;

procedure Tmainfrm.getjrebtnClick(Sender: TObject);
begin
     if opendlg.Execute then
     begin
     if opendlg.FileName[length(opendlg.FileName)] <> '\' then
     opendlg.FileName:= opendlg.FileName + '\';

     jrepath.Text:= opendlg.FileName;
     crec.jredir:= jrepath.Text;
     savecfg;
     end;

end;

procedure Tmainfrm.getndkbtnClick(Sender: TObject);
begin
     if opendlg.Execute then
     begin
     if opendlg.FileName[length(opendlg.FileName)] <> '\' then
     opendlg.FileName:= opendlg.FileName + '\';

     ndkpath.Text:= opendlg.FileName;
     crec.ndkdir:= ndkpath.Text;
     savecfg;
     end;

end;

procedure Tmainfrm.getsdkbtnClick(Sender: TObject);
begin
     if opendlg.Execute then
     begin
     if opendlg.FileName[length(opendlg.FileName)] <> '\' then
     opendlg.FileName:= opendlg.FileName + '\';

     sdkpath.Text:= opendlg.FileName;
     crec.sdkdir:= sdkpath.Text;
     savecfg;
     end;

end;

procedure Tmainfrm.savebtnClick(Sender: TObject);
begin
     savecfg;
end;

procedure Tmainfrm.showList;
var
   gl,
   t,
   i : integer;

begin
     pgrid.Clean;

     gl:= 1;
     t:= lst.Count;
     pgrid.RowCount:= t;
     x:= 0;

     for i:= 0 to t - 1 do
     if lst[i] = 'Installed packages:' then
     break;

     repeat
     progress.Append(lst[i]);

     if (first(1,lst[i]) = ' ') or
     (first(1,lst[i]) = '-') or
     (lst[i] = '') then
     begin
     inc(i);
     continue
     end
     else
     begin
     inst[x]:= lst[i];
     inc(x);
     end;

     inc(i);
     until lst[i] = 'Available Packages:';

     //pgrid.cells[0,gl]:= lst[i];
     inc(i,2);
     //inc(gl);

     while i < lst.Count do
     begin

     if lst[i] = '' then
     begin
     inc(i);

     if pgrid.RowCount <= gl then
     pgrid.RowCount:= gl;

     if lst[i] = 'Available Updates:' then
     break;

     tmp:= lst[i];
     pgrid.cells[0,gl]:= replace(tmp,';',' ');
     inc(i);

     tmp:= mirt(lst[i]);
     tmp1:= stripto(tmp,':');
     tmp:= mirt(tmp);
     pgrid.cells[2,gl]:= tmp;
     inc(i);

     tmp:= mirt(lst[i]);
     tmp1:= stripto(tmp,':');
     tmp1:= mirt(tmp);
     pgrid.cells[1,gl]:= mirt(tmp1);

     inc(gl);
     end;

     inc(i);
     end;

     pgrid.RowCount:= gl - 1;

     gl:= lst.Count;

     while i < gl do
     begin
     progress.Append(lst[i]);
     inc(i);
     end;

     if x > 0 then
     markInstalled(x);

     checkupdates;

end;

procedure Tmainfrm.markInstalled(ct : integer);
var
   st : integer;

   function instmatch(str : string) : boolean;
   var
      y : integer;

   begin
   for y:= 0 to ct - 1 do
   if replace(inst[y],';',' ') = str then
   begin
   result:= true;
   break;
   end;

   end;

begin
     x:= 0;

     if ct = 0 then
     exit;

     for st:= 0 to pgrid.RowCount - 1 do
     if instmatch(pgrid.cells[0,st]) then
     begin
     tmp:= '*' + pgrid.cells[0,st];
     pgrid.cells[0,st]:= tmp;
     end;

end;

procedure Tmainfrm.checkupdates;
begin
     i:= pgrid.rowcount div 2;
     x:= 0;

     while lst[i] <> 'Available Updates:' do
     inc(i);

     if lst[i] = 'Available Updates:' then
     updatebtn.Enabled:= true
     else
     updatebtn.Enabled:= false;

end;

procedure getcfg;
var
   cfile : file of config;

begin
     fillchar(crec,sizeof(crec),#0);
     assignfile(cfile,mydir + 'helper.cfg');

     try
     reset(cfile);
     read(cfile,crec);

     close(cfile);

     except
       savecfg;
     end;

end;

procedure savecfg;
var
   cfile : file of config;

begin
     assignfile(cfile,mydir + 'helper.cfg');

     if not directoryexists(mydir,false) then
     mkdir(mydir);

     try
     rewrite(cfile);
     write(cfile,crec);

     close(cfile);

     except
       showmessage('Could not create config file in ' + mydir);
     end;

end;

initialization

     mydir:= getenvironmentvariable('localappdata') + '\SDKhelper\'

end.

